package com.nisum.ascend.inventory.service;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.nisum.ascend.inventory.dto.Item;
import com.nisum.ascend.inventory.dto.WareHouseOrder;
import com.sun.tools.javac.jvm.Items;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.context.event.ApplicationStartedEvent;
import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Component;
import reactor.kafka.receiver.KafkaReceiver;

import java.util.List;

@Component
@Slf4j
public class KafkaListenerService {

    @Autowired
    private KafkaReceiver<String, String> kafkaReceiver;

    @Autowired
    private ProductService productService;

    @EventListener(ApplicationStartedEvent.class)
    public void consumeWarehouseOrderStatus() {
        kafkaReceiver.receive()
                .doOnNext(record -> log.info("record: {}", record))
                .doOnNext(record -> onWarehouseOrderStatusReceived(record.value()))
                .doOnError(throwable -> System.out.println(throwable.getMessage()))
                .subscribe();
    }

    private void onWarehouseOrderStatusReceived(String warehouseOrderString) {
        try {
            System.out.print(warehouseOrderString);
            ObjectMapper objectMapper = new ObjectMapper();
            WareHouseOrder warehouseOrder = objectMapper.readValue(warehouseOrderString, WareHouseOrder.class);
            List<Item> itemList = warehouseOrder.getOrderItems();
            for (Item item : itemList) {
                productService.updateProductInventoryBySku(String.valueOf(item.getItemSku()),warehouseOrder.getStatus(), item.getItemQuantity());
            }
//            log.info("Received this data: {}", warehouseOrder);
        } catch (Exception e) {
            log.error("error", e);
        }
    }
}
